@@ -1,15 +1,21 @@
 #!/usr/bin/env python2
 
-import sys, os
-import pwd, grp
-from gi.repository import Gtk, GObject, Gio, GdkPixbuf, AccountsService
+import os
+import sys
+import pwd
+import grp
 import gettext
 import shutil
-import PIL
-from PIL import Image
-from random import randint
 import re
 import subprocess
+from random import randint
+
+import PIL
+from PIL import Image
+import gi
+gi.require_version("Gtk", "3.0")
+gi.require_version("AccountsService", "1.0")
+from gi.repository import Gtk, GObject, Gio, GdkPixbuf, AccountsService
 
 gettext.install("cinnamon", "/usr/share/locale")
 
