@@ -1,10 +1,7 @@
 #!/usr/bin/env python2
 
 import sys
-sys.path.append('/usr/share/cinnamon/cinnamon-settings/bin')
-from SettingsWidgets import *
 import os
-from gi.repository import Gio, Gtk, GObject, Gdk, Pango, GLib
 import imtools
 import gettext
 import thread
@@ -12,9 +9,19 @@ import subprocess
 import tempfile
 import locale
 import time
-from xml.etree import ElementTree
-from PIL import Image
 import hashlib
+import mimetypes
+import pickle
+from io import BytesIO
+from xml.etree import ElementTree
+
+from PIL import Image
+import gi
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gio, Gtk, GObject, Gdk, Pango, GLib
+
+sys.path.append('/usr/share/cinnamon/cinnamon-settings/bin')
+from GSettingsWidgets import *
 
 gettext.install("cinnamon", "/usr/share/locale")
 
@@ -43,15 +50,6 @@ BACKGROUND_COLLECTION_TYPE_XML = "xml"
 
 (STORE_IS_SEPARATOR, STORE_ICON, STORE_NAME, STORE_PATH, STORE_TYPE) = range(5)
 
-
-def get_mimetype(filename):
-    """ Returns the mimetype of the file (eg. "image/png", "text/plain")
-
-    Throws CalledProcessError if the file does not exist
-    """
-    return subprocess.check_output(["file", "-bi", filename]).split(";")[0]
-
-
 # EXIF utility functions (source: http://stackoverflow.com/questions/4228530/pil-thumbnail-is-rotating-my-image)
 def flip_horizontal(im): return im.transpose(Image.FLIP_LEFT_RIGHT)
 def flip_vertical(im): return im.transpose(Image.FLIP_TOP_BOTTOM)
@@ -227,7 +225,7 @@ class Module:
             widget = GSettingsSwitch(_("Play backgrounds as a slideshow"), "org.cinnamon.desktop.background.slideshow", "slideshow-enabled")
             settings.add_row(widget)
 
-            widget = GSettingsSpinButton(_("Delay"), "org.cinnamon.desktop.background.slideshow", "delay", _("minutes"), 1, 120)
+            widget = GSettingsSpinButton(_("Delay"), "org.cinnamon.desktop.background.slideshow", "delay", _("minutes"), 1, 1440)
             settings.add_reveal_row(widget, "org.cinnamon.desktop.background.slideshow", "slideshow-enabled")
 
             widget = GSettingsSwitch(_("Play images in random order"), "org.cinnamon.desktop.background.slideshow", "random-order")
@@ -239,13 +237,13 @@ class Module:
             widget = GSettingsComboBox(_("Background gradient"), "org.cinnamon.desktop.background", "color-shading-type", BACKGROUND_COLOR_SHADING_TYPES, size_group=size_group)
             settings.add_reveal_row(widget, "org.cinnamon.desktop.background", "picture-options", PICTURE_OPTIONS_NEEDS_COLOR)
 
-            widget = GSettingsColorChooser(_("Gradient start color"), "org.cinnamon.desktop.background", "primary-color", size_group=size_group)
+            widget = GSettingsColorChooser(_("Gradient start color"), "org.cinnamon.desktop.background", "primary-color", legacy_string=True, size_group=size_group)
             settings.add_reveal_row(widget, "org.cinnamon.desktop.background", "picture-options", PICTURE_OPTIONS_NEEDS_COLOR)
 
             self._background_schema.connect("changed::picture-options", self.update_secondary_revealer)
             self._background_schema.connect("changed::color-shading-type", self.update_secondary_revealer)
 
-            widget = GSettingsColorChooser(_("Gradient end color"), "org.cinnamon.desktop.background", "secondary-color", size_group=size_group)
+            widget = GSettingsColorChooser(_("Gradient end color"), "org.cinnamon.desktop.background", "secondary-color", legacy_string=True, size_group=size_group)
             self.secondary_color_revealer = settings.add_reveal_row(widget)
 
             self.update_secondary_revealer(self._background_schema, None)
@@ -441,11 +439,7 @@ class Module:
                     files.sort()
                     for i in files:
                         filename = os.path.join(path, i)
-                        try:
-                            if get_mimetype(filename).startswith("image/"):
-                                picture_list.append({"filename": filename})
-                        except Exception, detail:
-                            print "Failed to detect mimetype for {}: {}".format(filename, detail)
+                        picture_list.append({"filename": filename})
                 elif type == BACKGROUND_COLLECTION_TYPE_XML:
                     picture_list += self.parse_xml_backgrounds_list(path)
 
@@ -533,13 +527,10 @@ class PixCache(object):
         self._data = {}
 
     def get_pix(self, filename, size=None):
-        try:
-            mimetype = get_mimetype(filename)
-            if not mimetype.startswith("image/"):
-                print "Not trying to convert %s : not a recognized image file" % filename
-                return None
-        except Exception, detail:
-            print "Failed to detect mimetype for %s: %s" % (filename, detail)
+        if filename is None:
+            return None
+        mimetype = mimetypes.guess_type(filename)[0]
+        if mimetype is None or not mimetype.startswith("image/"):
             return None
 
         if filename not in self._data:
@@ -552,30 +543,55 @@ class PixCache(object):
                 tmp_cache_path = GLib.get_user_cache_dir() + '/cs_backgrounds/'
                 if not os.path.exists(tmp_cache_path):
                     os.mkdir(tmp_cache_path)
-                cache_filename = tmp_cache_path + h
+                cache_filename = tmp_cache_path + h + "v2"
+
                 if os.path.exists(cache_filename):
-                    (width, height) = Image.open(filename).size
+                    # load from disk cache
+                    try:
+                        with open(cache_filename, "r") as cache_file:
+                            pix = pickle.load(cache_file)
+                        tmp_img = Image.open(BytesIO(pix[0]))
+                        pix[0] = self._image_to_pixbuf(tmp_img)
+                    except Exception, detail:
+                        print "Failed to load cache file: %s: %s" % (cache_filename, detail)
+                        pix = None
+
                 else:
                     if mimetype == "image/svg+xml":
+                        # rasterize svg with Gdk-Pixbuf and convert to PIL Image
                         tmp_pix = GdkPixbuf.Pixbuf.new_from_file(filename)
-                        tmp_fp, tmp_filename = tempfile.mkstemp()
-                        os.close(tmp_fp)
-                        tmp_pix.savev(tmp_filename, "png", [], [])
-                        img = Image.open(tmp_filename)
-                        os.unlink(tmp_filename)
+                        mode = "RGBA" if tmp_pix.props.has_alpha else "RGB"
+                        img = Image.frombytes(mode, (tmp_pix.props.width, tmp_pix.props.height),
+                                              tmp_pix.read_pixel_bytes().get_data(), "raw",
+                                              mode, tmp_pix.props.rowstride)
                     else:
                         img = Image.open(filename)
                         img = apply_orientation(img)
+
+                    # generate thumbnail
                     (width, height) = img.size
-                    if img.mode != 'RGB':
-                        img = img.convert('RGB')
+                    if img.mode != "RGB":
+                        if img.mode == "RGBA":
+                            bg_img = Image.new("RGBA", img.size, (255,255,255,255))
+                            img = Image.alpha_composite(bg_img, img)
+                        img = img.convert("RGB")
                     if size:
                         img.thumbnail((size, size), Image.ANTIALIAS)
                     img = imtools.round_image(img, {}, False, None, 3, 255)
-                    img = imtools.drop_shadow(img, 4, 4, background_color=(255, 255, 255, 0), shadow_color=0x444444, border=8, shadow_blur=3, force_background_color=False, cache=None)
-                    # Convert Image -> Pixbuf (save to file, GTK3 is not reliable for that)
-                    img.save(cache_filename, "png")
-                pix = [GdkPixbuf.Pixbuf.new_from_file(cache_filename), width, height]
+                    img = imtools.drop_shadow(img, 4, 4, background_color=(255, 255, 255, 0),
+                                              shadow_color=0x444444, border=8, shadow_blur=3,
+                                              force_background_color=False, cache=None)
+
+                    # save to disk cache
+                    try:
+                        png_bytes = BytesIO()
+                        img.save(png_bytes, "png")
+                        with open(cache_filename, "w") as cache_file:
+                            pickle.dump([png_bytes.getvalue(), width, height], cache_file, 2)
+                    except Exception, detail:
+                        print "Failed to save cache file: %s: %s" % (cache_filename, detail)
+
+                    pix = [self._image_to_pixbuf(img), width, height]
             except Exception, detail:
                 print "Failed to convert %s: %s" % (filename, detail)
                 pix = None
@@ -583,6 +599,13 @@ class PixCache(object):
                 self._data[filename][size] = pix
         return pix
 
+    # Convert RGBA PIL Image to Pixbuf
+    def _image_to_pixbuf(self, img):
+        return GdkPixbuf.Pixbuf.new_from_bytes(GLib.Bytes.new(img.tobytes()),
+                                               GdkPixbuf.Colorspace.RGB,
+                                               True, 8, img.width, img.height,
+                                               img.width * 4)
+
 PIX_CACHE = PixCache()
 
 
